/* win32run.c - December 12, 1997

  Simple wrapper to start Win32 GrADS executables. It must reside in the
directory Win32 GrADS was installed. You must copy this executable to
the following names, e.g.,

"gradsnc"       to run     gradsnc.dll

All command line arguments will be passed to the child process.
It is assumed a standard Win32 GrADS directory structure. If
Win32 GrADS is installed under /win32grads then:

GADDIR = /win32grads/dat
GASCRP = /win32grads/lib
DISPLAY = localhost:0.0

These environment variables are set iff they are not already defined.
The actual grads root directory is derived from the full pathname of the
executable, assuming the executable is under c:/win32grads

NOTES:

   1. This is a Windows application; is not meant to be portable.
   2. Compile with Microsoft Visual C++ Version
   3. It does not work well with GNU Win32 b18 due to a signal
      bug in execv() - to be fixed in b19.

 *   (c) 1997 Arlindo da Silva
 *
 *   Permission is granted to any individual or institution to use,
 *   copy, modify, or redistribute this software so long as it is not
 *   sold for profit, and provided this notice is retained.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <Windows.h>
// #include <Windows32/Functions.h>

static void setargv (int *argcPtr, char ***argvPtr);

#ifdef CONSOLE // Console application

int
main(int argc, char **argv)

#else         // Windows application

int APIENTRY
WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
    HINSTANCE hInstance;
    HINSTANCE hPrevInstance;
    LPSTR lpszCmdLine;
    int nCmdShow;

#endif

{

    FILE *o, *file;
    char * cmdLine;

    // char **argv;
    // int argc;

    char garoot[256];
    char gaddir[256];
    char  gagui[256];
    char gascrp[256];
    char gaudft[256];
    char display[256];
    char gaexec[256];
    char appl[256];
    char cmdl[256];
    char here[256];
    int i, ia, j, slash=0;

//  Retrieve the command line (Win32 API)
//  ------------------------------------
    cmdLine = GetCommandLine();

    // setargv(&argc, &argv);


//  Find GrADS root directory
//  -------------------------
    GetModuleFileName(NULL, cmdl, 256);   // Windows native call
    strcpy(garoot,cmdl);
    for(i=strlen(garoot)-1, ia=0; i>-1; i--)
    {
       if(garoot[i]=='/'||garoot[i]=='\\') slash++;
       if(slash==1 && !ia ) {
          ia = i+1;
          garoot[i] = '\0';
          break;
       }
    }


//  Find directory where this executable lives
//  ------------------------------------------
    strcpy(here,cmdl);
    here[ia-1] = '\0';


//  Find application name (long win32 name)
//  ---------------------------------------
    for(i=ia, j=0; i<strlen(cmdl); i++, j++) {
        if(cmdl[i]=='.') break;
        appl[j] = cmdl[i];
    }
    appl[j] = '\0';


//  Default environment strings
//  ---------------------------
    sprintf(gaddir,"GADDIR=%s/../dat", garoot);
    sprintf(gascrp,"GASCRP=%s/../lib", garoot);
    sprintf(gagui, "GAGUI=%s/../lib/sample2.gui", garoot);
    sprintf(gaudft,"GAUDFT=%s/../udf/udft", garoot);
    sprintf(display,"DISPLAY=localhost:0.0");


//  Set/reset environment variables
//  -------------------------------
    if (getenv("GADDIR")==NULL)      putenv(gaddir);
    if (getenv("GASCRP")==NULL)      putenv(gascrp);
    // if (getenv("GAUDT")==NULL)    putenv(gaudft);
    if (getenv("DISPLAY")==NULL)     putenv(display);


//  Some application names are aliased
//  ----------------------------------
    if ( !strcmp(appl,"Grads") || !strcmp(appl,"grads") 
                               || !strcmp(appl,"GRADS") ) {
          strcpy ( appl, "GradsNC" ); // this may not exist!
          sprintf(gaexec,"%s\\%s.dll", here, appl);
          file = fopen(gaexec,"r+b");
          if ( file == (FILE *)NULL ) {
               strcpy ( appl, "GradsC" ); // try classic
          } else {
               fclose(file);
          }
    }



    if ( !strcmp(appl,"GradsGUI") || !strcmp(appl,"GRADSGUI") ) {
         if (getenv("GAGUI")==NULL)  putenv(gagui);
         strcpy(appl,"GradsNC");
    }


//  Full path name of application
//  -----------------------------
    sprintf(gaexec,"%s/%s.dll", here, appl);


#ifndef __GNUWIN32__
// Turn / into \ for "DOS" compliancy
    for(i=strlen(gaexec)-1; i>-1; i--)
        if(gaexec[i]=='/') gaexec[i] = '\\';
#endif

//  Starting application
//  --------------------
    // printf("Starting %s...\n", gaexec);
    // system("env");

#ifdef __CYGWIN32__
    execv(gaexec,argv);
#else

    // execl(gaexec,cmdLine,NULL);
    // sprintf(gaexec,"%s/%s.dll %s", here, appl, cmdLine);
    // return system(gaexec);
    return(spawnl(_P_WAIT,gaexec,cmdLine,NULL));

#endif
}
